#include "opencv2/opencv.hpp"
#include "Halide.h"
#include "halide_image_io.h"

using namespace Halide;
using namespace Halide::Tools;

int main() {
  cv::Mat img = cv::imread("test.jpg",
                           cv::IMREAD_GRAYSCALE);
  cv::Mat result = img.clone();
  for (int y = 1; y < img.rows - 1; y += 8) {
    for (int x = 1; x < img.cols - 1; x += 8) {
      for (int y_inner = 0; y_inner < 8; y_inner++) {
        for (int x_inner = 0; x_inner < 8; x_inner++) {
          float sum = 0;
          for (int j = -1; j <= 1; j++) {
            for (int i = -1; i <= 1; i++) {
              sum += img.at<uint8_t>(y + y_inner + j, x + x_inner + i);
            }
          }
          result.at<uint8_t>(y + y_inner, x + x_inner) = sum / 9.;
        }
      }
    }
  }

  cv::imwrite("test_blur.jpg", result);


  Buffer<uint8_t> input = load_image("test.jpg");
  Func blur("blur");
  Var x("x"), y("y");
  blur(x, y) = (input(x, y) + input(x + 1, y) \
    + input(x + 2, y)) / 3;
  Buffer<uint8_t> output =
    blur.realize(input.width() - 2, input.height());
  save_image(output, "test_blur2.jpg");

  Var x_inner("x_inner"), x_outer("x_outer"), y_inner("y_inner"), y_outer("y_outer");
  blur.split(x, x_outer, x_inner, 4);
  blur.split(y, y_outer, y_inner, 4);
  blur.reorder(x_inner, y_inner, x_outer, y_outer);
  blur.vectorize(x_inner);
  blur.parallel(y_outer);

  blur.print_loop_nest();
  blur.compile_to_c("code.c", {}, "blur");
}
